@@ -61,6 +61,6 @@ impl CategoricalChunked {
         counts.rename("counts");
         let cols = vec![values.into_series(), counts.into_series()];
         let df = DataFrame::new_no_checks(cols);
-        df.sort(&["counts"], true)
+        df.sort(["counts"], true)
     }
 }
