@@ -5,7 +5,7 @@ use arrow::bitmap::Bitmap;
 use rayon::prelude::*;
 
 use crate::prelude::*;
-use crate::utils::get_supertype;
+use crate::utils::try_get_supertype;
 use crate::POOL;
 #[derive(Debug, Clone, PartialEq, Eq, Default)]
 pub struct Row<'a>(pub Vec<AnyValue<'a>>);
@@ -57,14 +57,14 @@ impl DataFrame {
     /// Create a new DataFrame from rows. This should only be used when you have row wise data,
     /// as this is a lot slower than creating the `Series` in a columnar fashion
     #[cfg_attr(docsrs, doc(cfg(feature = "rows")))]
-    pub fn from_rows_and_schema(rows: &[Row], schema: &Schema) -> Result<Self> {
+    pub fn from_rows_and_schema(rows: &[Row], schema: &Schema) -> PolarsResult<Self> {
         Self::from_rows_iter_and_schema(rows.iter(), schema)
     }
 
     /// Create a new DataFrame from an iterator over rows. This should only be used when you have row wise data,
     /// as this is a lot slower than creating the `Series` in a columnar fashion
     #[cfg_attr(docsrs, doc(cfg(feature = "rows")))]
-    pub fn from_rows_iter_and_schema<'a, I>(mut rows: I, schema: &Schema) -> Result<Self>
+    pub fn from_rows_iter_and_schema<'a, I>(mut rows: I, schema: &Schema) -> PolarsResult<Self>
     where
         I: Iterator<Item = &'a Row<'a>>,
     {
@@ -78,7 +78,7 @@ impl DataFrame {
             })
             .collect();
 
-        rows.try_for_each::<_, Result<()>>(|row| {
+        rows.try_for_each::<_, PolarsResult<()>>(|row| {
             for (value, buf) in row.0.iter().zip(&mut buffers) {
                 buf.add_fallible(value)?
             }
@@ -99,8 +99,8 @@ impl DataFrame {
     /// Create a new DataFrame from rows. This should only be used when you have row wise data,
     /// as this is a lot slower than creating the `Series` in a columnar fashion
     #[cfg_attr(docsrs, doc(cfg(feature = "rows")))]
-    pub fn from_rows(rows: &[Row]) -> Result<Self> {
-        let schema = rows_to_schema(rows, Some(50));
+    pub fn from_rows(rows: &[Row]) -> PolarsResult<Self> {
+        let schema = rows_to_schema_first_non_null(rows, Some(50));
         let has_nulls = schema
             .iter_dtypes()
             .any(|dtype| matches!(dtype, DataType::Null));
@@ -112,7 +112,7 @@ impl DataFrame {
         Self::from_rows_and_schema(rows, &schema)
     }
 
-    pub(crate) fn transpose_from_dtype(&self, dtype: &DataType) -> Result<DataFrame> {
+    pub(crate) fn transpose_from_dtype(&self, dtype: &DataType) -> PolarsResult<DataFrame> {
         let new_width = self.height();
         let new_height = self.width();
 
@@ -169,7 +169,7 @@ impl DataFrame {
 
     #[cfg_attr(docsrs, doc(cfg(feature = "rows")))]
     /// Transpose a DataFrame. This is a very expensive operation.
-    pub fn transpose(&self) -> Result<DataFrame> {
+    pub fn transpose(&self) -> PolarsResult<DataFrame> {
         let height = self.height();
         let width = self.width();
         if height == 0 || width == 0 {
@@ -238,14 +238,73 @@ pub fn coerce_data_type<A: Borrow<DataType>>(datatypes: &[A]) -> DataType {
     }
 
     let (lhs, rhs) = (datatypes[0].borrow(), datatypes[1].borrow());
-    get_supertype(lhs, rhs).unwrap_or(Utf8)
+    try_get_supertype(lhs, rhs).unwrap_or(Utf8)
 }
 
-/// Infer schema from rows.
-pub fn rows_to_schema(rows: &[Row], infer_schema_length: Option<usize>) -> Schema {
+fn is_nested_null(av: &AnyValue) -> bool {
+    match av {
+        AnyValue::Null => true,
+        AnyValue::List(s) => s.null_count() == s.len(),
+        #[cfg(feature = "dtype-struct")]
+        AnyValue::Struct(avs, _) => avs.iter().all(is_nested_null),
+        _ => false,
+    }
+}
+
+// nested dtypes that are all null, will be set as null leaf dtype
+fn infer_dtype_dynamic(av: &AnyValue) -> DataType {
+    match av {
+        AnyValue::List(s) if s.null_count() == s.len() => DataType::List(Box::new(DataType::Null)),
+        #[cfg(feature = "dtype-struct")]
+        AnyValue::Struct(avs, _) => DataType::Struct(
+            avs.iter()
+                .map(|av| {
+                    let dtype = infer_dtype_dynamic(av);
+                    Field::new("", dtype)
+                })
+                .collect(),
+        ),
+        av => av.into(),
+    }
+}
+
+/// Infer schema from rows and set the supertypes of the columns as column data type.
+pub fn rows_to_schema_supertypes(
+    rows: &[Row],
+    infer_schema_length: Option<usize>,
+) -> PolarsResult<Schema> {
+    // no of rows to use to infer dtype
+    let max_infer = infer_schema_length.unwrap_or(rows.len());
+
+    let mut dtypes: Vec<PlHashSet<DataType>> = vec![PlHashSet::with_capacity(4); rows[0].0.len()];
+
+    for row in rows.iter().take(max_infer) {
+        for (val, types_set) in row.0.iter().zip(dtypes.iter_mut()) {
+            let dtype = infer_dtype_dynamic(val);
+            types_set.insert(dtype);
+        }
+    }
+
+    dtypes
+        .into_iter()
+        .enumerate()
+        .map(|(i, types_set)| {
+            let dtype = types_set
+                .into_iter()
+                .map(Ok)
+                .fold_first_(|a, b| try_get_supertype(&a?, &b?))
+                .unwrap()?;
+            Ok(Field::new(format!("column_{}", i).as_ref(), dtype))
+        })
+        .collect::<PolarsResult<_>>()
+}
+
+/// Infer schema from rows and set the first no null type as column data type.
+pub fn rows_to_schema_first_non_null(rows: &[Row], infer_schema_length: Option<usize>) -> Schema {
     // no of rows to use to infer dtype
     let max_infer = infer_schema_length.unwrap_or(rows.len());
     let mut schema: Schema = (&rows[0]).into();
+
     // the first row that has no nulls will be used to infer the schema.
     // if there is a null, we check the next row and see if we can update the schema
 
@@ -255,10 +314,13 @@ pub fn rows_to_schema(rows: &[Row], infer_schema_length: Option<usize>) -> Schem
             .iter_dtypes()
             .enumerate()
             .filter_map(|(i, dtype)| {
-                if matches!(dtype, DataType::Null) {
-                    Some(i)
-                } else {
-                    None
+                // double check struct and list types types
+                // nested null values can be wrongly inferred by front ends
+                match dtype {
+                    DataType::Null | DataType::List(_) => Some(i),
+                    #[cfg(feature = "dtype-struct")]
+                    DataType::Struct(_) => Some(i),
+                    _ => None,
                 }
             })
             .collect();
@@ -266,8 +328,12 @@ pub fn rows_to_schema(rows: &[Row], infer_schema_length: Option<usize>) -> Schem
             break;
         } else {
             for i in nulls {
-                let dtype = (&row.0[i]).into();
-                schema.coerce_by_index(i, dtype).unwrap();
+                let val = &row.0[i];
+
+                if !is_nested_null(val) {
+                    let dtype = val.into();
+                    schema.coerce_by_index(i, dtype).unwrap();
+                }
             }
         }
     }
@@ -310,8 +376,6 @@ pub(crate) enum AnyValueBuffer<'a> {
     Float32(PrimitiveChunkedBuilder<Float32Type>),
     Float64(PrimitiveChunkedBuilder<Float64Type>),
     Utf8(Utf8ChunkedBuilder),
-    #[cfg(feature = "dtype-categorical")]
-    Categorical(CategoricalChunkedBuilder),
     All(DataType, Vec<AnyValue<'a>>),
 }
 
@@ -349,12 +413,24 @@ impl<'a> AnyValueBuffer<'a> {
             (Utf8(builder), AnyValue::Null) => builder.append_null(),
             // Struct and List can be recursive so use anyvalues for that
             (All(_, vals), v) => vals.push(v),
+
+            // dynamic types
+            (Float64(builder), av) => builder.append_value(av.extract()?),
+            (Int64(builder), av) => builder.append_value(av.extract()?),
+            (Utf8(builder), av) => match av {
+                AnyValue::Utf8(v) => builder.append_value(v),
+                AnyValue::Int64(v) => builder.append_value(&format!("{}", v)),
+                AnyValue::Float64(v) => builder.append_value(&format!("{}", v)),
+                AnyValue::Boolean(true) => builder.append_value("true"),
+                AnyValue::Boolean(false) => builder.append_value("false"),
+                _ => return None,
+            },
             _ => return None,
         };
         Some(())
     }
 
-    pub(crate) fn add_fallible(&mut self, val: &AnyValue<'a>) -> Result<()> {
+    pub(crate) fn add_fallible(&mut self, val: &AnyValue<'a>) -> PolarsResult<()> {
         self.add(val.clone()).ok_or_else(|| {
             PolarsError::ComputeError(format!("Could not append {:?} to builder; make sure that all rows have the same schema.\n\
             Or consider increasing the the 'schema_inference_length' argument.", val).into())
@@ -378,8 +454,6 @@ impl<'a> AnyValueBuffer<'a> {
             Float32(b) => b.finish().into_series(),
             Float64(b) => b.finish().into_series(),
             Utf8(b) => b.finish().into_series(),
-            #[cfg(feature = "dtype-categorical")]
-            Categorical(b) => b.finish().into_series(),
             All(dtype, vals) => Series::from_any_values_and_dtype("", &vals, &dtype).unwrap(),
         }
     }
@@ -407,15 +481,13 @@ impl From<(&DataType, usize)> for AnyValueBuffer<'_> {
             Float32 => AnyValueBuffer::Float32(PrimitiveChunkedBuilder::new("", len)),
             Float64 => AnyValueBuffer::Float64(PrimitiveChunkedBuilder::new("", len)),
             Utf8 => AnyValueBuffer::Utf8(Utf8ChunkedBuilder::new("", len, len * 5)),
-            #[cfg(feature = "dtype-categorical")]
-            Categorical(_) => AnyValueBuffer::Categorical(CategoricalChunkedBuilder::new("", len)),
             // Struct and List can be recursive so use anyvalues for that
             dt => AnyValueBuffer::All(dt.clone(), Vec::with_capacity(len)),
         }
     }
 }
 
-fn numeric_transpose<T>(cols: &[Series]) -> Result<DataFrame>
+fn numeric_transpose<T>(cols: &[Series]) -> PolarsResult<DataFrame>
 where
     T: PolarsNumericType,
     ChunkedArray<T>: IntoSeries,
@@ -520,7 +592,7 @@ mod test {
     use super::*;
 
     #[test]
-    fn test_transpose() -> Result<()> {
+    fn test_transpose() -> PolarsResult<()> {
         let df = df![
             "a" => [1, 2, 3],
             "b" => [10, 20, 30],
