@@ -27,57 +27,20 @@ pub use ndjson::*;
 #[cfg(feature = "parquet")]
 pub use parquet::*;
 use polars_arrow::prelude::QuantileInterpolOptions;
-#[cfg(any(feature = "parquet", feature = "csv-file", feature = "ipc"))]
-use polars_core::datatypes::PlHashMap;
 use polars_core::frame::explode::MeltArgs;
 use polars_core::frame::hash_join::JoinType;
 use polars_core::prelude::*;
 use polars_io::RowCount;
-#[cfg(feature = "serde")]
-use serde::{Deserialize, Serialize};
+pub use polars_plan::frame::{AllowedOptimizations, OptState};
+use polars_plan::global::FETCH_ROWS;
+#[cfg(any(feature = "ipc", feature = "parquet", feature = "csv-file"))]
+use polars_plan::logical_plan::collect_fingerprints;
+use polars_plan::logical_plan::optimize;
+use polars_plan::utils::{combine_predicates_expr, expr_to_leaf_column_names};
 
-use crate::logical_plan::optimizer::aggregate_pushdown::AggregatePushdown;
-#[cfg(any(feature = "parquet", feature = "csv-file", feature = "ipc"))]
-use crate::logical_plan::optimizer::file_caching::FileCacher;
-use crate::logical_plan::optimizer::predicate_pushdown::PredicatePushDown;
-use crate::logical_plan::optimizer::projection_pushdown::ProjectionPushDown;
-use crate::logical_plan::optimizer::simplify_expr::SimplifyExprRule;
-use crate::logical_plan::optimizer::stack_opt::{OptimizationRule, StackOptimizer};
-use crate::logical_plan::FETCH_ROWS;
+use crate::physical_plan::executors::Executor;
 use crate::physical_plan::state::ExecutionState;
-use crate::prelude::delay_rechunk::DelayRechunk;
-use crate::prelude::drop_nulls::ReplaceDropNulls;
-use crate::prelude::fast_projection::FastProjection;
-#[cfg(any(feature = "ipc", feature = "parquet", feature = "csv-file"))]
-use crate::prelude::file_caching::collect_fingerprints;
-#[cfg(any(feature = "ipc", feature = "parquet", feature = "csv-file"))]
-use crate::prelude::file_caching::find_column_union_and_fingerprints;
-use crate::prelude::simplify_expr::SimplifyBooleanRule;
-use crate::prelude::slice_pushdown_lp::SlicePushDown;
 use crate::prelude::*;
-use crate::utils::{combine_predicates_expr, expr_to_root_column_names};
-
-#[derive(Clone, Debug)]
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-pub struct JoinOptions {
-    pub allow_parallel: bool,
-    pub force_parallel: bool,
-    pub how: JoinType,
-    pub suffix: Cow<'static, str>,
-    pub slice: Option<(i64, usize)>,
-}
-
-impl Default for JoinOptions {
-    fn default() -> Self {
-        JoinOptions {
-            allow_parallel: true,
-            force_parallel: false,
-            how: JoinType::Left,
-            suffix: "_right".into(),
-            slice: None,
-        }
-    }
-}
 
 pub trait IntoLazy {
     fn lazy(self) -> LazyFrame;
@@ -116,39 +79,9 @@ impl From<LogicalPlan> for LazyFrame {
     }
 }
 
-#[derive(Copy, Clone)]
-/// State of the allowed optimizations
-pub struct OptState {
-    pub projection_pushdown: bool,
-    pub predicate_pushdown: bool,
-    pub type_coercion: bool,
-    pub simplify_expr: bool,
-    pub file_caching: bool,
-    pub aggregate_pushdown: bool,
-    pub slice_pushdown: bool,
-}
-
-impl Default for OptState {
-    fn default() -> Self {
-        OptState {
-            projection_pushdown: true,
-            predicate_pushdown: true,
-            type_coercion: true,
-            simplify_expr: true,
-            slice_pushdown: true,
-            // will be toggled by a scan operation such as csv scan or parquet scan
-            file_caching: false,
-            aggregate_pushdown: false,
-        }
-    }
-}
-
-/// AllowedOptimizations
-pub type AllowedOptimizations = OptState;
-
 impl LazyFrame {
     /// Get a hold on the schema of the current LazyFrame computation.
-    pub fn schema(&self) -> Result<SchemaRef> {
+    pub fn schema(&self) -> PolarsResult<SchemaRef> {
         let logical_plan = self.clone().get_plan_builder().build();
         logical_plan.schema().map(|schema| schema.into_owned())
     }
@@ -185,6 +118,8 @@ impl LazyFrame {
             // will be toggled by a scan operation such as csv scan or parquet scan
             file_caching: false,
             aggregate_pushdown: false,
+            #[cfg(feature = "cse")]
+            common_subplan_elimination: false,
         })
     }
 
@@ -212,6 +147,14 @@ impl LazyFrame {
         self
     }
 
+    /// Toggle common subplan elimination optimization on or off
+    #[cfg(feature = "cse")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "cse")))]
+    pub fn with_common_subplan_elimination(mut self, toggle: bool) -> Self {
+        self.opt_state.common_subplan_elimination = toggle;
+        self
+    }
+
     /// Toggle aggregate pushdown.
     pub fn with_aggregate_pushdown(mut self, toggle: bool) -> Self {
         self.opt_state.aggregate_pushdown = toggle;
@@ -230,9 +173,9 @@ impl LazyFrame {
     }
 
     /// Describe the optimized logical plan.
-    pub fn describe_optimized_plan(&self) -> Result<String> {
-        let mut expr_arena = Arena::with_capacity(512);
-        let mut lp_arena = Arena::with_capacity(512);
+    pub fn describe_optimized_plan(&self) -> PolarsResult<String> {
+        let mut expr_arena = Arena::with_capacity(64);
+        let mut lp_arena = Arena::with_capacity(64);
         let lp_top = self.clone().optimize(&mut lp_arena, &mut expr_arena)?;
         let logical_plan = node_to_lp(lp_top, &mut expr_arena, &mut lp_arena);
         Ok(logical_plan.describe())
@@ -337,7 +280,7 @@ impl LazyFrame {
             for (old, new) in existing2.iter().zip(new2.iter()) {
                 new_schema
                     .rename(old, new.to_string())
-                    .ok_or_else(|| PolarsError::NotFound(old.into()))?
+                    .ok_or_else(|| PolarsError::NotFound(old.to_string().into()))?
             }
             Ok(Arc::new(new_schema))
         };
@@ -409,15 +352,23 @@ impl LazyFrame {
         .map(
             move |mut df: DataFrame| {
                 let cols = df.get_columns_mut();
+                let mut removed_count = 0;
                 for (existing, new) in existing.iter().zip(new.iter()) {
-                    let idx_a = cols
-                        .iter()
-                        .position(|s| s.name() == existing.as_str())
-                        .unwrap();
-                    let idx_b = cols.iter().position(|s| s.name() == new.as_str()).unwrap();
-                    cols.swap(idx_a, idx_b);
+                    let idx_a = cols.iter().position(|s| s.name() == existing.as_str());
+                    let idx_b = cols.iter().position(|s| s.name() == new.as_str());
+
+                    match (idx_a, idx_b) {
+                        (Some(idx_a), Some(idx_b)) => {
+                            cols.swap(idx_a, idx_b);
+                        }
+                        // renamed columns are removed by predicate pushdown
+                        _ => {
+                            removed_count += 1;
+                            continue;
+                        }
+                    }
                 }
-                cols.truncate(cols.len() - existing.len());
+                cols.truncate(cols.len() - (existing.len() - removed_count));
                 Ok(df)
             },
             None,
@@ -532,7 +483,7 @@ impl LazyFrame {
     /// Note that the fetch does not guarantee the final number of rows in the DataFrame.
     /// Filter, join operations and a lower number of rows available in the scanned file influence
     /// the final number of rows.
-    pub fn fetch(self, n_rows: usize) -> Result<DataFrame> {
+    pub fn fetch(self, n_rows: usize) -> PolarsResult<DataFrame> {
         FETCH_ROWS.with(|fetch_rows| fetch_rows.set(Some(n_rows)));
         let res = self.collect();
         FETCH_ROWS.with(|fetch_rows| fetch_rows.set(None));
@@ -543,133 +494,11 @@ impl LazyFrame {
         self,
         lp_arena: &mut Arena<ALogicalPlan>,
         expr_arena: &mut Arena<AExpr>,
-    ) -> Result<Node> {
-        // get toggle values
-        let predicate_pushdown = self.opt_state.predicate_pushdown;
-        let projection_pushdown = self.opt_state.projection_pushdown;
-        let type_coercion = self.opt_state.type_coercion;
-        let simplify_expr = self.opt_state.simplify_expr;
-        let slice_pushdown = self.opt_state.slice_pushdown;
-
-        #[cfg(any(feature = "ipc", feature = "parquet", feature = "csv-file"))]
-        let agg_scan_projection = self.opt_state.file_caching;
-        let aggregate_pushdown = self.opt_state.aggregate_pushdown;
-
-        let logical_plan = self.get_plan_builder().build();
-
-        // gradually fill the rules passed to the optimizer
-        let opt = StackOptimizer {};
-        let mut rules: Vec<Box<dyn OptimizationRule>> = Vec::with_capacity(8);
-
-        // during debug we check if the optimizations have not modified the final schema
-        #[cfg(debug_assertions)]
-        let prev_schema = logical_plan.schema()?.into_owned();
-
-        let mut lp_top = to_alp(logical_plan, expr_arena, lp_arena)?;
-
-        // we do simplification
-        if simplify_expr {
-            rules.push(Box::new(SimplifyExprRule {}));
-        }
-
-        if projection_pushdown {
-            let projection_pushdown_opt = ProjectionPushDown {};
-            let alp = lp_arena.take(lp_top);
-            let alp = projection_pushdown_opt.optimize(alp, lp_arena, expr_arena)?;
-            lp_arena.replace(lp_top, alp);
-        }
-
-        if predicate_pushdown {
-            let predicate_pushdown_opt = PredicatePushDown::default();
-            let alp = lp_arena.take(lp_top);
-            let alp = predicate_pushdown_opt.optimize(alp, lp_arena, expr_arena)?;
-            lp_arena.replace(lp_top, alp);
-        }
-
-        // make sure its before slice pushdown.
-        if projection_pushdown {
-            rules.push(Box::new(FastProjection {}));
-        }
-        rules.push(Box::new(DelayRechunk {}));
-
-        if slice_pushdown {
-            let slice_pushdown_opt = SlicePushDown {};
-            let alp = lp_arena.take(lp_top);
-            let alp = slice_pushdown_opt.optimize(alp, lp_arena, expr_arena)?;
-
-            lp_arena.replace(lp_top, alp);
-
-            // expressions use the stack optimizer
-            rules.push(Box::new(slice_pushdown_opt));
-        }
-        if type_coercion {
-            rules.push(Box::new(TypeCoercionRule {}))
-        }
-        // this optimization removes branches, so we must do it when type coercion
-        // is completed
-        if simplify_expr {
-            rules.push(Box::new(SimplifyBooleanRule {}));
-        }
-
-        if aggregate_pushdown {
-            rules.push(Box::new(AggregatePushdown::new()))
-        }
-
-        // make sure that we do that once slice pushdown
-        // and predicate pushdown are done. At that moment
-        // the file fingerprints are finished.
-        #[cfg(any(feature = "ipc", feature = "parquet", feature = "csv-file"))]
-        if agg_scan_projection {
-            // we do this so that expressions are simplified created by the pushdown optimizations
-            // we must clean up the predicates, because the agg_scan_projection
-            // uses them in the hashtable to determine duplicates.
-            let simplify_bools =
-                &mut [Box::new(SimplifyBooleanRule {}) as Box<dyn OptimizationRule>];
-            lp_top = opt.optimize_loop(simplify_bools, expr_arena, lp_arena, lp_top);
-
-            // scan the LP to aggregate all the column used in scans
-            // these columns will be added to the state of the AggScanProjection rule
-            let mut file_predicate_to_columns_and_count = PlHashMap::with_capacity(32);
-            find_column_union_and_fingerprints(
-                lp_top,
-                &mut file_predicate_to_columns_and_count,
-                lp_arena,
-                expr_arena,
-            );
-
-            let rule = FileCacher::new(file_predicate_to_columns_and_count);
-            // its important that we do it now
-            // because typo coercion will change the predicates and there for
-            lp_top = opt.optimize_loop(
-                &mut [Box::new(rule) as Box<dyn OptimizationRule>],
-                expr_arena,
-                lp_arena,
-                lp_top,
-            );
-        }
-
-        rules.push(Box::new(ReplaceDropNulls {}));
-
-        lp_top = opt.optimize_loop(&mut rules, expr_arena, lp_arena, lp_top);
-
-        // during debug we check if the optimizations have not modified the final schema
-        #[cfg(debug_assertions)]
-        {
-            // only check by names because we may supercast types.
-            assert_eq!(
-                prev_schema.iter_names().collect::<Vec<_>>(),
-                lp_arena
-                    .get(lp_top)
-                    .schema(lp_arena)
-                    .iter_names()
-                    .collect::<Vec<_>>()
-            );
-        };
-
-        Ok(lp_top)
+    ) -> PolarsResult<Node> {
+        optimize(self.logical_plan, self.opt_state, lp_arena, expr_arena)
     }
 
-    fn prepare_collect(self) -> Result<(ExecutionState, Box<dyn Executor>)> {
+    fn prepare_collect(self) -> PolarsResult<(ExecutionState, Box<dyn Executor>)> {
         let file_caching = self.opt_state.file_caching;
         let mut expr_arena = Arena::with_capacity(256);
         let mut lp_arena = Arena::with_capacity(128);
@@ -706,14 +535,14 @@ impl LazyFrame {
     /// use polars_core::prelude::*;
     /// use polars_lazy::prelude::*;
     ///
-    /// fn example(df: DataFrame) -> Result<DataFrame> {
+    /// fn example(df: DataFrame) -> PolarsResult<DataFrame> {
     ///     df.lazy()
     ///       .groupby([col("foo")])
     ///       .agg([col("bar").sum(), col("ham").mean().alias("avg_ham")])
     ///       .collect()
     /// }
     /// ```
-    pub fn collect(self) -> Result<DataFrame> {
+    pub fn collect(self) -> PolarsResult<DataFrame> {
         let (mut state, mut physical_plan) = self.prepare_collect()?;
         let out = physical_plan.execute(&mut state);
         #[cfg(debug_assertions)]
@@ -731,7 +560,7 @@ impl LazyFrame {
     //// of each node that is executed.
     ////
     //// The units of the timings are microseconds.
-    pub fn profile(self) -> Result<(DataFrame, DataFrame)> {
+    pub fn profile(self) -> PolarsResult<(DataFrame, DataFrame)> {
         let (mut state, mut physical_plan) = self.prepare_collect()?;
         state.time_nodes();
         let out = physical_plan.execute(&mut state)?;
@@ -827,16 +656,39 @@ impl LazyFrame {
             .map(|e| e.clone().into())
             .collect::<Vec<_>>();
         let opt_state = self.get_opt_state();
-        LazyGroupBy {
-            logical_plan: self.logical_plan,
-            opt_state,
-            keys,
-            maintain_order: false,
-            dynamic_options: None,
-            rolling_options: None,
+
+        #[cfg(feature = "dynamic_groupby")]
+        {
+            LazyGroupBy {
+                logical_plan: self.logical_plan,
+                opt_state,
+                keys,
+                maintain_order: false,
+                dynamic_options: None,
+                rolling_options: None,
+            }
+        }
+
+        #[cfg(not(feature = "dynamic_groupby"))]
+        {
+            LazyGroupBy {
+                logical_plan: self.logical_plan,
+                opt_state,
+                keys,
+                maintain_order: false,
+            }
         }
     }
 
+    /// Create rolling groups based on a time column.
+    ///
+    /// Also works for index values of type Int32 or Int64.
+    ///
+    /// Different from a [`dynamic_groupby`] the windows are now determined by the
+    /// individual values and are not of constant intervals. For constant intervals use
+    /// *groupby_dynamic*
+    #[cfg(feature = "dynamic_groupby")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dynamic_groupby")))]
     pub fn groupby_rolling<E: AsRef<[Expr]>>(
         self,
         by: E,
@@ -853,6 +705,23 @@ impl LazyFrame {
         }
     }
 
+    /// Group based on a time value (or index value of type Int32, Int64).
+    ///
+    /// Time windows are calculated and rows are assigned to windows. Different from a
+    /// normal groupby is that a row can be member of multiple groups. The time/index
+    /// window could be seen as a rolling window, with a window size determined by
+    /// dates/times/values instead of slots in the DataFrame.
+    ///
+    /// A window is defined by:
+    ///
+    /// - every: interval of the window
+    /// - period: length of the window
+    /// - offset: offset of the window
+    ///
+    /// The `by` argument should be empty `[]` if you don't want to combine this
+    /// with a ordinary groupby on these keys.
+    #[cfg(feature = "dynamic_groupby")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dynamic_groupby")))]
     pub fn groupby_dynamic<E: AsRef<[Expr]>>(
         self,
         by: E,
@@ -869,7 +738,7 @@ impl LazyFrame {
         }
     }
 
-    /// Similar to groupby, but order of the DataFrame is maintained.
+    /// Similar to [`groupby`], but order of the DataFrame is maintained.
     pub fn groupby_stable<E: AsRef<[IE]>, IE: Into<Expr> + Clone>(self, by: E) -> LazyGroupBy {
         let keys = by
             .as_ref()
@@ -877,13 +746,27 @@ impl LazyFrame {
             .map(|e| e.clone().into())
             .collect::<Vec<_>>();
         let opt_state = self.get_opt_state();
-        LazyGroupBy {
-            logical_plan: self.logical_plan,
-            opt_state,
-            keys,
-            maintain_order: true,
-            dynamic_options: None,
-            rolling_options: None,
+
+        #[cfg(feature = "dynamic_groupby")]
+        {
+            LazyGroupBy {
+                logical_plan: self.logical_plan,
+                opt_state,
+                keys,
+                maintain_order: true,
+                dynamic_options: None,
+                rolling_options: None,
+            }
+        }
+
+        #[cfg(not(feature = "dynamic_groupby"))]
+        {
+            LazyGroupBy {
+                logical_plan: self.logical_plan,
+                opt_state,
+                keys,
+                maintain_order: true,
+            }
         }
     }
 
@@ -1183,7 +1066,7 @@ impl LazyFrame {
         name: Option<&'static str>,
     ) -> LazyFrame
     where
-        F: 'static + Fn(DataFrame) -> Result<DataFrame> + Send + Sync,
+        F: 'static + Fn(DataFrame) -> PolarsResult<DataFrame> + Send + Sync,
     {
         let opt_state = self.get_opt_state();
         let lp = self
@@ -1198,6 +1081,12 @@ impl LazyFrame {
         Self::from_logical_plan(lp, opt_state)
     }
 
+    pub(crate) fn map_private(self, function: FunctionNode) -> LazyFrame {
+        let opt_state = self.get_opt_state();
+        let lp = self.get_plan_builder().map_private(function).build();
+        Self::from_logical_plan(lp, opt_state)
+    }
+
     /// Add a new column at index 0 that counts the rows.
     ///
     /// # Warning
@@ -1312,7 +1201,9 @@ pub struct LazyGroupBy {
     opt_state: OptState,
     keys: Vec<Expr>,
     maintain_order: bool,
+    #[cfg(feature = "dynamic_groupby")]
     dynamic_options: Option<DynamicGroupOptions>,
+    #[cfg(feature = "dynamic_groupby")]
     rolling_options: Option<RollingGroupOptions>,
 }
 
@@ -1340,6 +1231,7 @@ impl LazyGroupBy {
     /// }
     /// ```
     pub fn agg<E: AsRef<[Expr]>>(self, aggs: E) -> LazyFrame {
+        #[cfg(feature = "dynamic_groupby")]
         let lp = LogicalPlanBuilder::from(self.logical_plan)
             .groupby(
                 Arc::new(self.keys),
@@ -1350,6 +1242,11 @@ impl LazyGroupBy {
                 self.rolling_options,
             )
             .build();
+
+        #[cfg(not(feature = "dynamic_groupby"))]
+        let lp = LogicalPlanBuilder::from(self.logical_plan)
+            .groupby(Arc::new(self.keys), aggs, None, self.maintain_order)
+            .build();
         LazyFrame::from_logical_plan(lp, self.opt_state)
     }
 
@@ -1358,7 +1255,7 @@ impl LazyGroupBy {
         let keys = self
             .keys
             .iter()
-            .flat_map(|k| expr_to_root_column_names(k).into_iter())
+            .flat_map(|k| expr_to_leaf_column_names(k).into_iter())
             .collect::<Vec<_>>();
 
         self.agg([col("*").exclude(&keys).head(n).list().keep_name()])
@@ -1370,7 +1267,7 @@ impl LazyGroupBy {
         let keys = self
             .keys
             .iter()
-            .flat_map(|k| expr_to_root_column_names(k).into_iter())
+            .flat_map(|k| expr_to_leaf_column_names(k).into_iter())
             .collect::<Vec<_>>();
 
         self.agg([col("*").exclude(&keys).tail(n).keep_name()])
@@ -1381,8 +1278,18 @@ impl LazyGroupBy {
     /// this as materializing the `DataFrame` is very expensive.
     pub fn apply<F>(self, f: F, schema: SchemaRef) -> LazyFrame
     where
-        F: 'static + Fn(DataFrame) -> Result<DataFrame> + Send + Sync,
+        F: 'static + Fn(DataFrame) -> PolarsResult<DataFrame> + Send + Sync,
     {
+        #[cfg(feature = "dynamic_groupby")]
+        let options = GroupbyOptions {
+            dynamic: None,
+            rolling: None,
+            slice: None,
+        };
+
+        #[cfg(not(feature = "dynamic_groupby"))]
+        let options = GroupbyOptions { slice: None };
+
         let lp = LogicalPlan::Aggregate {
             input: Box::new(self.logical_plan),
             keys: Arc::new(self.keys),
@@ -1390,11 +1297,7 @@ impl LazyGroupBy {
             schema,
             apply: Some(Arc::new(f)),
             maintain_order: self.maintain_order,
-            options: GroupbyOptions {
-                dynamic: None,
-                rolling: None,
-                slice: None,
-            },
+            options,
         };
         LazyFrame::from_logical_plan(lp, self.opt_state)
     }
