@@ -14,17 +14,19 @@ use crate::physical_plan::exotic::{prepare_eval_expr, prepare_expression_for_con
 use crate::physical_plan::state::ExecutionState;
 use crate::prelude::*;
 
-impl PhysicalAggExpr for Expr {
-    fn evaluate<'a>(&self, df: &DataFrame, groups: &'a GroupsProxy) -> Result<Series> {
+struct PivotExpr(Expr);
+
+impl PhysicalAggExpr for PivotExpr {
+    fn evaluate<'a>(&self, df: &DataFrame, groups: &'a GroupsProxy) -> PolarsResult<Series> {
         let state = ExecutionState::new();
         let dtype = df.get_columns()[0].dtype();
-        let phys_expr = prepare_expression_for_context("", self, dtype, Context::Aggregation)?;
+        let phys_expr = prepare_expression_for_context("", &self.0, dtype, Context::Aggregation)?;
         phys_expr
             .evaluate_on_groups(df, groups, &state)
             .map(|mut ac| ac.aggregated())
     }
 
-    fn root_name(&self) -> Result<&str> {
+    fn root_name(&self) -> PolarsResult<&str> {
         Ok("")
     }
 }
@@ -36,7 +38,7 @@ pub fn pivot<I0, S0, I1, S1, I2, S2>(
     columns: I2,
     agg_expr: Expr,
     sort_columns: bool,
-) -> Result<DataFrame>
+) -> PolarsResult<DataFrame>
 where
     I0: IntoIterator<Item = S0>,
     S0: AsRef<str>,
@@ -52,7 +54,7 @@ where
         values,
         index,
         columns,
-        PivotAgg::Expr(Arc::new(expr)),
+        PivotAgg::Expr(Arc::new(PivotExpr(expr))),
         sort_columns,
     )
 }
@@ -64,7 +66,7 @@ pub fn pivot_stable<I0, S0, I1, S1, I2, S2>(
     columns: I2,
     agg_expr: Expr,
     sort_columns: bool,
-) -> Result<DataFrame>
+) -> PolarsResult<DataFrame>
 where
     I0: IntoIterator<Item = S0>,
     S0: AsRef<str>,
@@ -80,7 +82,7 @@ where
         values,
         index,
         columns,
-        PivotAgg::Expr(Arc::new(expr)),
+        PivotAgg::Expr(Arc::new(PivotExpr(expr))),
         sort_columns,
     )
 }
