@@ -20,7 +20,7 @@ impl PhysicalExpr for LiteralExpr {
     fn as_expression(&self) -> Option<&Expr> {
         Some(&self.1)
     }
-    fn evaluate(&self, _df: &DataFrame, _state: &ExecutionState) -> Result<Series> {
+    fn evaluate(&self, _df: &DataFrame, _state: &ExecutionState) -> PolarsResult<Series> {
         use LiteralValue::*;
         let s = match &self.0 {
             #[cfg(feature = "dtype-i8")]
@@ -74,7 +74,7 @@ impl PhysicalExpr for LiteralExpr {
                 }
             },
             Utf8(v) => Utf8Chunked::full("literal", v, 1).into_series(),
-            #[cfg(all(feature = "temporal", feature = "dtype-datetime"))]
+            #[cfg(feature = "temporal")]
             DateTime(ndt, tu) => {
                 use polars_core::chunked_array::temporal::conversion::*;
                 let timestamp = match tu {
@@ -86,7 +86,7 @@ impl PhysicalExpr for LiteralExpr {
                     .into_datetime(*tu, None)
                     .into_series()
             }
-            #[cfg(all(feature = "temporal", feature = "dtype-duration"))]
+            #[cfg(feature = "temporal")]
             Duration(v, tu) => {
                 let duration = match tu {
                     TimeUnit::Milliseconds => v.num_milliseconds(),
@@ -126,7 +126,7 @@ impl PhysicalExpr for LiteralExpr {
         df: &DataFrame,
         groups: &'a GroupsProxy,
         state: &ExecutionState,
-    ) -> Result<AggregationContext<'a>> {
+    ) -> PolarsResult<AggregationContext<'a>> {
         let s = self.evaluate(df, state)?;
         Ok(AggregationContext::from_literal(s, Cow::Borrowed(groups)))
     }
@@ -135,7 +135,7 @@ impl PhysicalExpr for LiteralExpr {
         Some(self)
     }
 
-    fn to_field(&self, _input_schema: &Schema) -> Result<Field> {
+    fn to_field(&self, _input_schema: &Schema) -> PolarsResult<Field> {
         let dtype = self.0.get_datatype();
         Ok(Field::new("literal", dtype))
     }
@@ -154,7 +154,7 @@ impl PartitionedAggregation for LiteralExpr {
         df: &DataFrame,
         _groups: &GroupsProxy,
         state: &ExecutionState,
-    ) -> Result<Series> {
+    ) -> PolarsResult<Series> {
         self.evaluate(df, state)
     }
 
@@ -163,7 +163,7 @@ impl PartitionedAggregation for LiteralExpr {
         partitioned: Series,
         _groups: &GroupsProxy,
         _state: &ExecutionState,
-    ) -> Result<Series> {
+    ) -> PolarsResult<Series> {
         Ok(partitioned)
     }
 }
